package bk;

import java.util.ArrayList;

import javax.swing.JTree;

import animation.AlgorithmStage;
import animation.PseudoCodeNode;
import animation.Memory.MemoryType;
import animation.Memory.ReadOnlyMemory;
import codelines.AbstractForLoop;
import codelines.DeclareVariable;
import codelines.FunctionDefinition;
import graph.LayeredGraphNode;
import lib.TextLayoutHelper;

/**
 * The stage of the BK node placement algorithm where the blocks are computed.
 * @author kolja
 *
 */
public class BlockCalc implements AlgorithmStage {

    @Override
    public PseudoCodeNode createPseudocodeTree( JTree tree ) {
        String[] vars = { "graph", "L", "v", "r", "neighbors", "layout" };
        PseudoCodeNode root = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "function calculateBlockGraph( graph, layout )", vars ), tree, new FunctionDefinition( new String[]{ "graph", "layout" } ) );
        root.add( new PseudoCodeNode(TextLayoutHelper.setupPseudoCode( "L = graph.getContainedLayers();", vars), tree, new DeclareVariable<ArrayList<ArrayList<LayeredGraphNode>>>( "L" ) {
            @Override
            protected ArrayList<ArrayList<LayeredGraphNode>> value(ReadOnlyMemory m) {
                return m.<LayeredGraphNode>read( "graph", MemoryType.LOCAL ).getContainedLayers();
            }
        } ) );
        PseudoCodeNode layerLoop = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "for i=layout.contains('DOWN') ? 0 : |L|-1 to layout.contains('DOWN') ? |L|-1 : 0 do", vars ), tree, new AbstractForLoop<Integer>( "i" ) {
            @Override
            protected Integer begin(ReadOnlyMemory m) {
                if( m.<String>read( "layout", MemoryType.LOCAL ).contains( "DOWN" ) )
                    return 0;
                return m.<ArrayList<ArrayList<LayeredGraphNode>>>read( "L", MemoryType.LOCAL ).size() - 1;
            }
            @Override
            protected Integer step(ReadOnlyMemory m) {
                if( m.<String>read( "layout", MemoryType.LOCAL ).contains( "DOWN" ) )
                    return m.<Integer>read( "i", MemoryType.LOCAL ) + 1;
                return m.<Integer>read( "i", MemoryType.LOCAL ) - 1;
            }
            @Override
            protected boolean condition(ReadOnlyMemory m) {
                if( m.<String>read( "layout", MemoryType.LOCAL ).contains( "DOWN" ) )
                    return m.<Integer>read( "i", MemoryType.LOCAL ) <= m.<ArrayList<ArrayList<LayeredGraphNode>>>read( "L", MemoryType.LOCAL ).size() - 1;
                return m.<Integer>read( "i", MemoryType.LOCAL ) >= 0;
            }
        });
        root.add( layerLoop );
        layerLoop.add( new PseudoCodeNode(TextLayoutHelper.setupPseudoCode( "r = layout.contains('RIGHT') ? -1 : INFINITY;", vars), tree, new DeclareVariable<Double>( "r" ) {
            @Override
            protected Double value(ReadOnlyMemory m) {
                if( m.<String>read( "layout", MemoryType.LOCAL ).contains( "RIGHT" ) )
                    return -1.0;
                return Double.POSITIVE_INFINITY;
            }
        } ) );
        PseudoCodeNode nodeLoop = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "for k=layout.contains('RIGHT') ? 0 : |L[i]|-1 to layout.contains('RIGHT') ? |L[i]|-1 : 0 do", vars ), tree, new AbstractForLoop<Integer>( "i" ) {
            @Override
            protected Integer begin( ReadOnlyMemory m) {
                if( m.<String>read( "layout", MemoryType.LOCAL ).contains( "RIGHT" ) )
                    return 0;
                return m.<ArrayList<ArrayList<LayeredGraphNode>>>read( "L", MemoryType.LOCAL ).get( m.read( "i", MemoryType.LOCAL ) ).size() - 1;
            }
            @Override
            protected Integer step( ReadOnlyMemory m) {
                if( m.<String>read( "layout", MemoryType.LOCAL ).contains( "RIGHT" ) )
                    return m.<Integer>read( "k", MemoryType.LOCAL ) + 1;
                return m.<Integer>read( "k", MemoryType.LOCAL ) - 1;
            }
            @Override
            protected boolean condition( ReadOnlyMemory m) {
                if( m.<String>read( "layout", MemoryType.LOCAL ).contains( "RIGHT" ) )
                    return m.<Integer>read( "k", MemoryType.LOCAL ) <= m.<ArrayList<ArrayList<LayeredGraphNode>>>read( "L", MemoryType.LOCAL ).get( m.read( "i", MemoryType.LOCAL ) ).size() - 1;
                return m.<Integer>read( "k", MemoryType.LOCAL ) >= 0;
            }
        });
        layerLoop.add( nodeLoop );
        return root;
    }
}
